/*      	 
 * Copyright 2002 Slicer/HackersQuest (slicer@hackersquest.org)
 *
 * This file is part of Odin's Eye.
 *
 * Odin's Eye is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Odin's Eye is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import java.io.*;

public class Connection extends JPanel implements Runnable {
  JLabel xl;
  JLabel yl;
  JLabel zl;

  public int x;
  public int y;
  public int z;

  boolean running;

  String hostport;

  long lastupdate;

  boolean stale;

  Socket con;

  MapEditor editor;

  public Connection(MapEditor edit) {
    JLabel xlab=new JLabel("X");
    JLabel ylab=new JLabel("Y");
    JLabel zlab=new JLabel("Z");
    
    xl=new JLabel();
    yl=new JLabel();
    zl=new JLabel();

    con = null;
    stale = false;
    lastupdate = 0;
    hostport=new String("somehost:1234");

    x=y=z=0;
    updatePosition();
    updateBorder();

    setLayout(new GridLayout(1,6));
    add(xlab);
    add(xl);
    add(ylab);
    add(yl);
    add(zlab);
    add(zl);
  
    editor=edit;
  }

  public void updateBorder() {
    boolean nstale;

    nstale=false;

    if ((con == null) || (! con.isConnected())) {
      nstale = true;
    } else if ((System.currentTimeMillis() - lastupdate) > 5000) {
      nstale = true;
    }

    if (nstale != stale) {
      String title;
      stale = nstale;
      if (stale)
         title="Connection (stale)";
      else
         title="Connection (fresh)";
      setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), title));
    }
  }

  public void updatePosition() {
    xl.setText(String.valueOf(x));
    yl.setText(String.valueOf(y));
    zl.setText(String.valueOf(z));
    if (editor != null)
      editor.updateConnection();
  }

  void err(String msg) {
    JOptionPane.showMessageDialog(null, msg, "Error in Link", JOptionPane.ERROR_MESSAGE);
    con = null;
    updateBorder();
  }

  public boolean connect(String nval) {
    if (nval != null) {
      try {
        URL u=new URL("http://"+nval);
        String host;
        int port;
        host=u.getHost();
        port=u.getPort();
        if ((host == null) || (host.length() < 1) || (port == -1) || (port == 23)) {
           throw(new MalformedURLException());
        }
        hostport=host+":"+port;
        con=new Socket(host, port);
        con.setKeepAlive(true);
        con.setSoTimeout(10000);
        new Thread(this).start();
        return true;
      } catch (MalformedURLException e) {
        err(nval + " isn't a valid host/port");
      } catch (UnknownHostException e) {
        err(nval + " cannot be found");
      } catch (SocketException e) {
        err("Socket options aren't working");
      } catch (IOException e) {
        err("Failed to create socket");
      }
    }
    return false;
  }

  public boolean connect() {
    String nval=JOptionPane.showInputDialog("Enter the host and port to connect to",hostport);
    return connect(nval);
  }

  public void disconnect() {
    if ((con != null) && (con.isConnected())) {
      try {
        Socket c=con;
        con=null;
        c.close();
      } catch (IOException e) {
      }
    }
    updateBorder();
  }

  public void run() {
    Socket origcon=con;
    BufferedReader in;
    String line;
    String vals[];
    String error;

    try {
      in = new BufferedReader(new InputStreamReader(origcon.getInputStream()));
      line=in.readLine();
    } catch (IOException e) {
      err("Cannot read from socket");
      return;
    }

    try {
      if (Integer.valueOf(line).intValue() != 1) {
        err("Remote version is "+line+", but editor is made for version 1");
        return;
      }
    } catch (NumberFormatException e) {
      err("Weird data from socket");
      return;
    }

    while (origcon == con) {
      try {
        line=in.readLine();
        if (line == null) 
          continue;
        vals=line.split(" ");
        x=Integer.valueOf(vals[0]).intValue();
        y=Integer.valueOf(vals[1]).intValue();
        z=Integer.valueOf(vals[2]).intValue();
        lastupdate=System.currentTimeMillis();
        updatePosition();
        updateBorder();
      } catch (SocketTimeoutException e) {
        updateBorder();        
      } catch (SocketException e) {
        if (con != origcon) 
          return;
        err("Connection closed");
        return;
      } catch (IOException e) {
        err("Socket failure");
        return;
      } catch (NumberFormatException e) {
        err("Weird data from socket");
        return;
      }
    }
  }
}
