/*      	 
 * Copyright 2002 Slicer/HackersQuest (slicer@hackersquest.org)
 *
 * This file is part of Odin's Eye.
 *
 * Odin's Eye is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Odin's Eye is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.imageio.*;
import java.io.*;
import java.awt.image.*;
import java.util.zip.Inflater;
import java.util.zip.DataFormatException;
import java.text.*;

/*
 * The MPK class got some of it's inspiration from dempak.c, which
 * was was
 * Copyright (c) 2001, Oliver Jowett <oliver@randomly.org>.
 * dempak.c can be found at http://www.randomly.org/projects/mapper/
 */

class MPKException extends Exception {
  public MPKException(String message) {
    super(message);
  }
}

class MPK {
  HashMap entries;
  byte file[];
  int lastofs;
  Inflater inf;

  MPK(InputStream is) throws IOException, DataFormatException, MPKException {
    entries=new HashMap();

    Convert.s("Reading MPK file");

    ByteArrayOutputStream bos;

    byte header[]=new byte[4];
    is.read(header);
    if (! new String(header).equals("MPAK")) {
      throw(new MPKException("Missing Header"));
    }
    is.skip(17);

    bos=new ByteArrayOutputStream();
    int l;
    byte b[]=new byte[1024];

    while ((l=is.read(b))>0) {
      bos.write(b, 0, l);
    }

    file=bos.toByteArray();

    lastofs = 0;

    Convert.d();

    Convert.s("Setting up decompression ");

    inf=new Inflater();

    String name=new String(getBlock());

    Convert.d();
    Convert.s("Decompressing contents of "+name);
    byte directory[]=getBlock();
    
    if ((directory.length % 0x11c) != 0) {
      throw(new MPKException("Wrong directory length"));
    }

    int dirofs=0;
    while (dirofs < directory.length) {
      StringBuffer sb=new StringBuffer();
      for (int i=dirofs;(directory[i] != 0) && (i<directory.length);i++)
        sb.append((char) directory[i]);
      String fname=new String(sb).toLowerCase();
      entries.put(fname, getBlock());
      dirofs+=0x11c;
    }

    Convert.d();
  }

  byte []getBlock() throws DataFormatException {
    inf.setInput(file, lastofs, file.length-lastofs);
    ByteArrayOutputStream bos=new ByteArrayOutputStream();
    byte b[]=new byte[1024];
    int l;

    do {
      l=inf.inflate(b);
      if (l != 0)
        bos.write(b, 0, l);
    } while (l==b.length);

    lastofs=file.length - inf.getRemaining();
    inf.reset();

    return bos.toByteArray();
  }

  InputStream getFile(String name) {
    Object o=entries.get(name);
    if (o == null)
      return null;
    byte [] b=(byte []) o;
    return new ByteArrayInputStream(b);
  }
}

class WinIni {
  HashMap sections;

  WinIni(InputStream is) throws IOException {

    HashMap sect;

    sect = null;

    sections = new HashMap();

    BufferedReader r=new BufferedReader(new InputStreamReader(is));
    String l;
    while ((l=r.readLine())!=null) {
      l=l.toLowerCase();
      int o;
      o=l.indexOf(";");
      if (o != -1)
       l=l.substring(0,o);
      l=l.trim();
      if (l.length() < 1)
        continue;
      if (l.startsWith("[") && l.endsWith("]")) {
        l=l.substring(1,l.length()-1);
        sect=new HashMap();
        sections.put(l, sect);
      } else if (sect == null) {
        continue;
      } else {
        String []opts=l.split("=");
        if (opts.length != 2) 
          continue;
        sect.put(opts[0],opts[1]);
      }
    }
  }

  String getString(String section, String key, String def) {
    HashMap sect=(HashMap) sections.get(section);
    if (sect == null) {
      System.out.println("Section "+section+" not found");
      return def;
    }
    String value=(String) sect.get(key);
    if (value == null) {
       System.out.println("Key "+section+"/"+key+" not found");
       return def;
    }
    return value;
  }

  int getInt(String section, String key, int def) {
    String s=getString(section, key, null);
    if (s == null)
      return def;
    try {
      return Integer.valueOf(s).intValue();
    } catch (NumberFormatException e) {
      return def;
    }
  }

  int []getInts(String section, String key) {
    String s=getString(section, key, null);
    if (s == null)
      return null;
    String []sp=s.split(",");
    int r[]=new int[sp.length];
    try {
      for(int i=0;i<sp.length;i++) 
        r[i]=Integer.valueOf(sp[i]).intValue();
      return r;
    } catch (NumberFormatException e) {
      return null;
    }
  }

  String []getSections() {
    return (String []) sections.keySet().toArray(new String[0]);
  }
}


public class Convert {
  static int depth = 0;
  static boolean donestate = true;

  public static void s(String msg) {
    int i;

    if (! donestate)
      System.out.println("");

    for (i=0; i < depth; i++) {
      System.out.print("  ");
    }

    System.out.print(msg);
    System.out.print(" ");
    int l=msg.length();
    for (i=l;i<60-depth*2;i++) 
      System.out.print(".");
    System.out.flush();

    depth ++;
    donestate = false;
  }

  public static void prefin() {
    depth--;
    if (! donestate) {
      donestate = true;
      return;
    }
    for(int i=0; i < depth+1; i++) {
      System.out.print("  ");
    }
    System.out.print("...");
  }  

  public static void d() {
    prefin();
    System.out.println(" Done");
  }

  public static void f(Exception e) {
    prefin();
    System.out.println(" Failed: "+e.getMessage());
  }


  public static BufferedImage getTexture(MPK texfile) {
    BufferedImage bi=new BufferedImage(2048, 2048, BufferedImage.TYPE_INT_RGB);
    Graphics2D g=bi.createGraphics();
    for(int y=0;y<8;y++) {
      for(int x=0;x<8;x++) {
        String fname="tex0"+x+"-0"+y+".bmp";
        InputStream in;
        in=texfile.getFile(fname);
        if (in == null)
           return null;
        Image i=BMPLoader.read(in);
        g.drawImage(i, 256*x, 256*y, null);
      }
    }
    g.dispose();
    return bi;
  }

  public static BufferedImage getPCXImage(InputStream is) {
    Image i=PcxReader.loadImage(is);
    BufferedImage bi=new BufferedImage(i.getWidth(null),i.getHeight(null), BufferedImage.TYPE_INT_RGB);
    Graphics2D g=bi.createGraphics();
    g.drawImage(i, 0, 0, null);
    g.dispose();
    return bi;
  }

  public static int []getElevation(WinIni conf, MPK datafile) throws Exception {
    int offscale=48;
    int terrscale=8;

    int offs[]=PcxReader.load8bitImage(datafile.getFile("offset.pcx"));
    int terr[]=PcxReader.load8bitImage(datafile.getFile("terrain.pcx"));
    offscale=conf.getInt("terrain","offsetfactor",48);
    terrscale=conf.getInt("terrain","scalefactor",8);

    if (offs.length != terr.length) 
      throw(new Exception("Offset map and Terrain map are different size"));

    if ((offs.length != 65536) || (terr.length != 65536)) 
      throw(new Exception("Offset or terrain map is invalid size"));

    int elev[]=new int[256*256];

    for (int i=0;i<256*256;i++) {
      elev[i]=offs[i]*offscale+terr[i]*terrscale;
    } 

    return elev;
  }

  public static double getZ(double []elev, int x, int y) {
    int xo=x / 32;
    int yo=y / 32;
    return elev[yo * 2048 + xo];
  }

  /*
   * This WAS a pretty accurate scaler with good positioning
   * of resampled pixels.. But alas, that didn't match the
   * quick and dirty sub-linear interpolation used in the client
   * so water edges were off :(
   */

  public static double[]rescaleBig(double []in) throws Exception {
    if (in.length != 256*256)
      throw(new Exception("Scale input isn't 256*256"));

    double out[]=new double[2048*2048];
    for(int y=0;y<256;y++) {
      int yb = y;
      for (int x=0;x<2048;x++) {
        double v;
        if (x >= 2040) {
          v=in[y * 256+255];
        } else {
          int xb=(x) / 8;
          int xm=(x) % 8;
          v=in[y * 256 + xb] * (8 - xm) + in[y * 256 + xb+1] * xm;
          v=v / 8;
        }
        for(int i=0;i<8;i++) {
          out[(y * 8 + i)*2048+x]=v;
        }
      }
    }
    for(int y=0;y<2039;y+=8) {
      for(int x=0;x<2048;x++) {
        int bofs=y*2048+x;
        double a=out[bofs];
        double b=out[bofs+8*2048];
        for(int i=0;i<8;i++) {
          out[bofs + 2048 * i]=(a * (8-i) + b * i) / 8;
        }
      }
    }
    return out;
  }

  public static double[] divMap(int []in) {
    double out[]=new double[in.length];
    for(int i=0;i<in.length;i++) {
      out[i]=in[i];
//      out[i]=out[i] / 32.0;
    }
    return out;
  }

  /* 
   * I originally had a quick and dirty bumping, but then Sparr
   * pointed me at the implementation found at
   * www.ugcs.caltech.edu/~dsimpson/graphac.c
   * which I copied
   */

  public static double []lightMap(double elev[]) {
    double lightVector[]=new double[3];

    lightVector[0]=1.0;
    lightVector[1]=-1.0;
    lightVector[2]=1.0;
    
    double v[]=new double[3];

    int rgb[]=new int[3];

    double light[]=new double[256*256];

    for(int y=0;y<256;y++) {
      for(int x=0;x<256;x++) {
       	v[0] = 0.0; 
       	v[1] = 0.0;
       	v[2] = 0.0;  
       	if ((x < 256 - 1) && (y < 256 - 1)) {
            v[0] -= elev[y*256 + x + 1] - elev[y*256+x];
            v[1] -= elev[(y + 1) * 256 +  x] - elev[y*256+x];
            v[2] += 12.0;
       	}
       	if ((x > 0) && (y < 256 - 1)) {
            v[0] += elev[y * 256 + x - 1] - elev[y*256+x];
            v[1] -= elev[(y + 1)*256 + x] - elev[y*256+x];
            v[2] += 12.0;
       	}
	if ((x > 0) && (y > 0)) {
            v[0] += elev[y*256 + x - 1] - elev[y*256+x];
            v[1] += elev[(y - 1) * 256 + x] - elev[y*256+x];
            v[2] += 12.0;
       	}
       	if ((x < 256 - 1) && (y > 0)) {
            v[0] -= elev[y * 256 + x + 1] - elev[y*256+x];
            v[1] += elev[(y - 1) * 256 + x] - elev[y*256+x];
            v[2] += 12.0;
        } 
        light[y*256+x] = (((lightVector[0] * v[0] + lightVector[1] * v[1] + lightVector[2] * v[2]) /
            Math.sqrt((lightVector[0] * lightVector[0] + lightVector[1] * lightVector[1] + lightVector[2] * lightVector[2]) *
            (v[0] * v[0] + v[1] * v[1] + v[2] * v[2]))) * 128.0 + 128.0) * 2.25 + 0.0;

      }
    }
    return light;
  }

  public static BufferedImage bump(BufferedImage img, double []light) {
    int b[]=new int[2048*2048];
    int buff[]=img.getRGB(0, 0, 2048, 2048, b, 0, 2048);
    int rgb[]=new int[3];

    for (int p=0;p<2048*2048;p++) {
        int val=buff[p];
        rgb[0]=(val>>16) & 255;
        rgb[1]=(val>>8) & 255;
        rgb[2]=val & 255;

        for (int i=0;i<3;i++) {
          double r=rgb[i];
          r=(r * light[p]) / 256.0;
          if (r > 255.0)
            r=255.0;
          else if (r < 0.0)
            r=0.0;
          rgb[i]=(int)r;
        }
        buff[p]=(rgb[0]<<16) | (rgb[1]<<8) | rgb[2];
    }
    img.setRGB(0, 0, 2048, 2048, buff, 0, 2048);

    return img;
  }

  public static BufferedImage light(BufferedImage img, double light) {
    int bu[]=new int[img.getWidth() * img.getHeight()];
    int buff[]=img.getRGB(0, 0, img.getWidth(), img.getHeight(), bu, 0, img.getWidth());
    double r, g, b;
    int val;

    for (int p=0;p<bu.length;p++) {
        val=buff[p];
        r=(val >> 16) & 255;
        g=(val >> 8) & 255;
        b=(val & 255);
        r=(r * light);
        g=(g * light);
        b=(b * light);
        buff[p]=(((int)r)<<16) | (((int)g)<<8) | ((int)b);
    }
    img.setRGB(0, 0, img.getWidth(), img.getHeight(), buff, 0, img.getWidth());

    return img;
  }

  public static BufferedImage water(BufferedImage img, WinIni conf, int numwater, double []elev, int[] watertexture, int[] lavatexture) {
    int b[]=new int[2048*2048];
    int buff[]=img.getRGB(0, 0, 2048, 2048, b, 0, 2048);
    BufferedImage w=new BufferedImage(2048, 2048, BufferedImage.TYPE_INT_RGB);
    Graphics2D g=w.createGraphics();

    NumberFormat f=NumberFormat.getInstance();
    f.setMinimumIntegerDigits(2);

    for(int s=0;s<numwater;s++) {
      String sect="river"+f.format(s);
      int points=conf.getInt(sect,"bankpoints",0);
      if (points > 0) {
        g.setColor(Color.black);
        g.fillRect(0,0,2048,2048);
        g.setColor(Color.white);
        Polygon p=new Polygon();
        for(int j=0;j<points;j++) {
          int []left=conf.getInts(sect,"left"+f.format(j));
          p.addPoint(left[0]*8,left[1]*8);
        }
        for(int j=0;j<points;j++) {
          int []right=conf.getInts(sect,"right"+f.format(points-(j+1)));
          p.addPoint(right[0]*8,right[1]*8);
        }
        g.fill(p);
        int by[]=new int[2048 * 2048];
        int byff[]=w.getRGB(0, 0, 2048, 2048, by, 0, 2048);

        int text[];
        int col=conf.getInt(sect, "color", 0xff0000);
        String ttype=conf.getString(sect, "texture", "watery.tga");
        if (ttype.equals("lava.tga"))
          text=lavatexture;
        else
          text=watertexture;

        int height=conf.getInt(sect, "height", 1000);
        for(int y=0;y<2048;y++) {
          for(int x=0;x<2048;x++) {
            int o=y * 2048 + x;
            if (((byff[o] & 0xff) > 20) && (elev[o]<=height)) {
              int texofs=((y % 256)*256)+(x%256);
              int red=(((buff[o] >> 16) & 0xff) + ((text[texofs] >> 16) & 0xff))/2;
              int green=(((buff[o] >> 8) & 0xff) + ((text[texofs] >> 8) & 0xff))/2;
              int blue=((buff[o] & 0xff) + (text[texofs] & 0xff))/2;
              buff[o]=(red<<16)|(green<<8)|blue;
            }
          }
        }
      }
    }    
    g.dispose();
    img.setRGB(0, 0, 2048, 2048, buff, 0, 2048);
    return img;
  }

  public static int []divide(int []in, int w) {
    int x, y;
    int nw = w >> 1;
    int res[]=new int[nw*nw];

    for(int i=0;i<nw*nw;i++) 
      res[i]=0;

    for(y=0;y<w;y++) {
      for(x=0;x<w;x++) {
        res[(y >> 1) * nw + (x >> 1)]+=((in[y*w+x])>>2) & 0x3f3f3f;
      }
    }
    return res;
  }

  public static BufferedImage divide(BufferedImage img, int divlevel) {
    int b[]=new int[2048*2048];
    int buff[]=img.getRGB(0, 0, 2048, 2048, b, 0, 2048);

    int w = 2048;

    for(int i=1;i<divlevel;i++) {
      buff=divide(buff, (w >> (i-1)));
    }

    w = w >> (divlevel-1);

    BufferedImage res=new BufferedImage(w, w, BufferedImage.TYPE_INT_RGB);
    res.setRGB(0, 0, w, w, buff, 0, w);
    return res;
  }

  public static void main(String args[]) {
    if (args.length < 4) {
      System.out.println("Arguments are : <basepath> <divfactor> <light> <zone #1> [zone #2] [zone #3]");
      return;
    }

    String basepath=args[0];

    int divfactor;
    double light;

    try {
      divfactor=Integer.valueOf(args[1]).intValue();
    } catch (Exception e) {
      divfactor=3;
    }

    try {
      light=Double.valueOf(args[2]).doubleValue();
    } catch (Exception e) {
      light=1.0;
    }

    int []lava=new int[256*256];
    int []water=new int[256*256];

    try {
      BufferedImage waterimg=ImageIO.read(new File(basepath+"/zones/textures/watery.png"));
      waterimg.getRGB(0, 0, 256, 256, water, 0, 256);
    } catch (Exception e) {
      e.printStackTrace();
      System.out.println("Failed to read water texture -- Setting to blue");
      for(int i=0;i<256*256;i++) 
        water[i]=0x00ffff;
    }

    try {
      BufferedImage lavaimg=ImageIO.read(new File(basepath+"/zones/textures/lava.png"));
      lavaimg.getRGB(0, 0, 256, 256, lava, 0, 256);
    } catch (Exception e) {
      System.out.println("Failed to read lava texture -- Setting to red");
      for(int i=0;i<256*256;i++) 
        lava[i]=0xff2000;
    }
 
    for (int i=3;i<args.length;i++) {
      String z=args[i];
      s("Converting zone "+z);
      String basedir=basepath+"/zones/zone"+z+"/";
      File texfile=new File(basedir, "tex"+z+".mpk");
      File datfile=new File(basedir, "dat"+z+".mpk");
      File outfile=new File("zone"+z+".png");
      BufferedInputStream is;
      MPK dat, tex;
      try {
        s("Reading data file");
        is=new BufferedInputStream(new FileInputStream(datfile));
        dat=new MPK(is);
        d();
        s("Reading texture file");
        is=new BufferedInputStream(new FileInputStream(texfile));  
        tex=new MPK(is);
        d();
        s("Getting background texture");
        BufferedImage texture=getTexture(tex);
        d();
        s("Reading zone data");
        WinIni conf=new WinIni(dat.getFile("sector.dat"));
        d();
        s("Bumping map");
        int elev[]=getElevation(conf, dat);
        texture=bump(texture, rescaleBig(lightMap(divMap(elev))));
        d();
        int numwater=conf.getInt("waterdefs","num",0);
        if (numwater != 0) {
          s("Adding watertextures");
          texture=water(texture, conf, numwater, rescaleBig(divMap(elev)), water, lava);
          d();
        }
        if (divfactor != 1) {
          s("Reducing image size");
          texture=divide(texture, divfactor);
          d();
        }
        if (light != 1.0) {
          s("Applying light");
          texture=light(texture, light);
          d();
        }
        s("Saving computed texture to "+outfile.getName());
        ImageIO.write(texture, "png", outfile);
        d();
        d();
      } catch (Exception e) {
        e.printStackTrace();
        f(e);
        continue;
      }
    }
  }
}
