/*      	 
 * Copyright 2002 Slicer/HackersQuest (slicer@hackersquest.org)
 *
 * This file is part of Odin's Eye.
 *
 * Odin's Eye is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Odin's Eye is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class Map extends JPanel implements MouseListener {
  LinkedList elements;
  MapElement elem;
  String title;

  double xtrans[];
  double ytrans[];

  int xbase, ybase;

  int x, y;
  int width;
  int rot;

  MapEditor editor;
  Rectangle bounds;

  public Map(MapEditor edit) {
    elem=null;
    elements=new LinkedList();
    xtrans=new double[2];
    ytrans=new double[2];
    editor=edit;
    bounds = new Rectangle();
    home();
    addMouseListener(this);
  }

  public int transx(int x, int y) {
    return (int)(xtrans[0] * (x-xbase) + xtrans[1] * (y-ybase));
  }
  
  public int transy(int x, int y) {
    return (int)(ytrans[0] * (x-xbase) + ytrans[1] * (y-ybase));
  }

  public MapElement getSelected() {
    return elem;
  }

  public void setSelected(MapElement e) {
    elem=e;
    if (editor != null)
      editor.updateSelected();
  }

  public void wipe() {
    elements.clear();
  }

  public void home() {
    x= 0x8000;
    y= 0x8000;
    width = 0x8000;
  }
 
  public void move(int dir) {
    double amnt = width / 2;
    x+=(int)(Math.cos(Math.PI * dir / 2)*amnt);
    y-=(int)(Math.sin(Math.PI * dir / 2)*amnt);
  }

  public void zoom(int dir) {
    width = (int)(width * Math.pow(2, dir));
  }

  public void clean() {
    MapElement objs[]=(MapElement []) elements.toArray(new MapElement[0]);
    for(int i=0;i<objs.length;i++) {
      if (! objs[i].cleanAndValidate()) {
        elements.remove(objs[i]);
      }
    }

    TreeSet ts=new TreeSet();
    ts.addAll(elements);
    elements=new LinkedList(ts);
  }

  public void setColor(Color c) {
    if (elem == null)
      return;
    elem.setColor(c);
  }

  public void setFilled(boolean state) {
    if ((elem == null) || ! (elem instanceof MapElementLine))
      return;
    MapElementLine l=(MapElementLine) elem;
    l.setFilled(state);
  }

  public void deleteObject() {
    if (elem == null)
      return;
    elements.remove(elem);
    elem=null;
    repaint();
  }

  public void renameObject() {
    if (elem == null)
      return;
    String s=getObjName(elem);
    if (s != null) {
      elem.setTitle(s);
      repaint();
    }
  }

  public void lineDelete() {
    if ((elem == null) || ! (elem instanceof MapElementLine))
      return;
    MapElementLine l=(MapElementLine) elem;
    l.deleteSelected();
    updateSelectedNode();
  }

  public void lineFirst() {
    if ((elem == null) || ! (elem instanceof MapElementLine))
      return;
    MapElementLine l=(MapElementLine) elem;
    l.gotoFirst();
    updateSelectedNode();
  }

  public void lineLast() {
    if ((elem == null) || ! (elem instanceof MapElementLine))
      return;
    MapElementLine l=(MapElementLine) elem;
    l.gotoLast();
    updateSelectedNode();
  }

  public void lineFlatten() {
    if ((elem == null) || ! (elem instanceof MapElementLine))
      return;
    MapElementLine l=(MapElementLine) elem;
    l.flatten(editor.con.z);
  }

  public void linePoint() {
    if ((elem == null) || ! (elem instanceof MapElementLine))
      return;
    MapElementLine l=(MapElementLine) elem;
    l.add(editor.con.x,editor.con.y,editor.con.z);
    updateSelectedNode();
  }    

  public void lineNew() {
    elem=new MapElementLine();
    elem.setColor(editor.color);
    elements.add(elem);
  }

  public void pointNew() {
    String s;
    s=getObjName(null);
    if (s == null)
      return;
    elem=new MapElementPoint(editor.con.x,editor.con.y,editor.con.z);
    elem.setTitle(s);
    elements.add(elem);
  }

  public void simplify() {
    updateSelectedNode();
  }

  String getObjName(MapElement elem) {
    String oval=new String();
    if (elem != null)
      oval=elem.getTitle();
    String val=JOptionPane.showInputDialog(this, "What's the name of the element?", oval);
    if ((val != null) && (val.length() > 0))
      return val;
    return null;
  }

  public void retrans(int w, int h) {
    double xscale;
    double yscale;

    xscale=w;
    yscale=h;
 
    xbase = x-width;
    ybase = y-width;

    xscale=xscale / (2 * width);
    yscale=yscale / (2 * width);

    xtrans[0]=xscale;
    xtrans[1]=0.0;
    ytrans[1]=yscale;
    ytrans[0]=0.0;

    bounds=new Rectangle();
    bounds.add(x-width,y-width);
    bounds.add(x+width,y-width);
    bounds.add(x-width,y+width);
    bounds.add(x+width,y+width);
  }

  public void paintComponent(Graphics gr) {
    Dimension d=getSize();
    Graphics2D g=(Graphics2D) gr;

    int w, h;
    w=d.width;
    h=d.height;

    g.setColor(Color.black);
    g.fillRect(0, 0, w, h);

    retrans(w, h);

    if (elements.size() != 0) {
      MapElement elems[]=(MapElement []) elements.toArray(new MapElement[0]);
      for(int i=0;i<elems.length;i++) {
        if (elems[i].visible(bounds))
          elems[i].paint(g, this);
      }
    }

    g.setColor(Color.gray.darker());

    int xv, yv;
    xv=transx(editor.con.x,editor.con.y);
    yv=transy(editor.con.x,editor.con.y);

    g.drawLine(xv, 0, xv, h);
    g.drawLine(0, yv, w, yv);
  }

  public void updateSelectedNode() {
    if (elem instanceof MapElementLine) {
      MapElementLine l=(MapElementLine) elem;
      if (l.node != null) {
        int vx, vy;
        vx=l.getNodeX();
  	vy=l.getNodeY();
        if (! bounds.contains(vx, vy)) {
          x=vx;
          y=vy;
        }
      }
    }
  }

  void err(String msg) {
    JOptionPane.showMessageDialog(null, msg, "Error in Map Operations", JOptionPane.ERROR_MESSAGE);
  }
  
  public void read(File f) {
    BufferedReader in;
    String line;
    MapElement elm;

    elements.clear();
    elem=null;

    try {
      in = new BufferedReader(new FileReader(f));
    } catch (FileNotFoundException e) {
      err("File could not be read!");
      return;
    }
    try {
      line=in.readLine();
      while ((line=in.readLine())!=null) {
        elm=MapElement.fromString(line);
        if (elm != null) {
          elements.add(elm);
        } else {
          System.out.println("Line "+line+" failed conversion");
        }
      }
    } catch (IOException e) {
      return;
    }
  }

  public void write(File f) {
    PrintWriter out;
    String line;
    ArrayList al;

    clean();
    
    try {
      out=new PrintWriter(new BufferedWriter(new FileWriter(f)));
    } catch (IOException e) {
      err("File "+f.getAbsolutePath()+" could not be written!");
      return;
    }
    al=new ArrayList();
    line=f.getName().replaceAll(".map","");
    String cleanname=line.replaceAll("_"," ");
    al.add(cleanname);
    al.add(line);
    for(int i=0;i<6;i++) 
      al.add(String.valueOf(0));
    line=MapElement.join(al);
    out.println(line);
    MapElement objs[]=(MapElement []) elements.toArray(new MapElement[0]);
    for(int i=0;i<objs.length;i++)
      out.println(objs[i].toString());
    out.flush();
    out.close();
  }

  public void mouseClicked(MouseEvent e) {
    int cx, cy;
    cx=e.getX();
    cy=e.getY();

    double vx, vy;

    vx=cx / xtrans[0];
    vy=cy / ytrans[1];
    vx+=xbase;
    vy+=ybase;

    cx=(int) vx;
    cy=(int) vy;

    double best=-1.0;
    double d;

    elem = null;

    MapElement objs[]=(MapElement [])elements.toArray(new MapElement[0]);
    for(int i=0;i<objs.length;i++) {
      d=objs[i].clickAt(new Point(cx, cy));
      if ((best < 0.0) || (d < best)) {
        best=d;
        elem=objs[i];
      }
    }
    setSelected(elem);
    repaint();
  }

  public void mouseEntered(MouseEvent e) {
  }

  public void mouseExited(MouseEvent e) {
  }

  public void mousePressed(MouseEvent e) {
  }

  public void mouseReleased(MouseEvent e) {
  }
}
