/*      	 
 * Copyright 2002 Slicer/HackersQuest (slicer@hackersquest.org)
 *
 * This file is part of Odin's Eye.
 *
 * Odin's Eye is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Odin's Eye is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class MapEditor extends JFrame implements ActionListener, ItemListener, KeyListener {
  JMenuItem menuFileNew, menuFileOpen, menuFileSave, menuFileExit, menuMapClean, menuMapSimple, menuObjectDelete, menuObjectRename;
  JMenuItem menuLinkConnect, menuLinkDisconnect;
  JMenuItem menuLineNew, menuLinePoint, menuLineDelete, menuLineFirst, menuLineLast, menuLineFlatten;
  JCheckBoxMenuItem menuLineFilled;
  JMenuItem menuPointNew;
  JRadioButtonMenuItem menuObjectColors[];
  static Color colors[]={Color.red, Color.blue, Color.green, Color.white, Color.gray, Color.cyan, Color.yellow, Color.magenta, new Color(165,42,42)}; 
  static String colornames[]={"Red","Blue","Green","White","Gray","Cyan","Yellow","Magenta", "Brown"};
  Connection con;
  Color color;
  Map map;
  File lastfile;

  public MapEditor() {
    setTitle("Map Editor");
    setSize(750,550);

    JMenuBar menubar=new JMenuBar();
    setJMenuBar(menubar);
    
    JMenu menu;
    JMenuItem item;

    color = Color.white;

    menu=new JMenu("File");
    menu.setMnemonic(KeyEvent.VK_F);
    menubar.add(menu);

    menuFileNew=new JMenuItem("New", KeyEvent.VK_N);
    menuFileNew.addActionListener(this);
    menu.add(menuFileNew);
    
    menuFileOpen=new JMenuItem("Open...", KeyEvent.VK_O);
    menuFileOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
    menuFileOpen.addActionListener(this);
    menu.add(menuFileOpen);

    menuFileSave=new JMenuItem("Save...", KeyEvent.VK_S);
    menuFileSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    menuFileSave.addActionListener(this);
    menu.add(menuFileSave);

    menu.addSeparator();
    menuFileExit=new JMenuItem("Exit", KeyEvent.VK_X);
    menuFileExit.addActionListener(this);
    menu.add(menuFileExit);

    menu=new JMenu("Link");
    menu.setMnemonic(KeyEvent.VK_I);
    menubar.add(menu);

    menuLinkConnect=new JMenuItem("Connect", KeyEvent.VK_C);
    menuLinkConnect.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.ALT_MASK));
    menuLinkConnect.addActionListener(this);
    menu.add(menuLinkConnect);

    menuLinkDisconnect=new JMenuItem("Disconnect", KeyEvent.VK_D);
    menuLinkDisconnect.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.ALT_MASK));
    menuLinkDisconnect.addActionListener(this);
    menuLinkDisconnect.setEnabled(false);
    menu.add(menuLinkDisconnect);

    menu=new JMenu("Map");
    menu.setMnemonic(KeyEvent.VK_M);
    menubar.add(menu);
   
    menuMapClean=new JMenuItem("Clean", KeyEvent.VK_C);
    menuMapClean.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, 0));
    menuMapClean.addActionListener(this);
    menu.add(menuMapClean);

    menuMapSimple=new JMenuItem("Find useless node", KeyEvent.VK_S);
    menuMapSimple.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, 0));
    menuMapSimple.addActionListener(this);
    menu.add(menuMapSimple);

    menu=new JMenu("Object");
    menu.setMnemonic(KeyEvent.VK_O);
    menubar.add(menu);

    ButtonGroup bg=new ButtonGroup();
    
    JMenu submenu=new JMenu("Color");
    submenu.setMnemonic(KeyEvent.VK_C);
    menu.add(submenu);

    menuObjectColors=new JRadioButtonMenuItem[colors.length];

    for(int i=0;i<colors.length;i++) {
      menuObjectColors[i]=new JRadioButtonMenuItem(colornames[i]);
      menuObjectColors[i].addItemListener(this);
      submenu.add(menuObjectColors[i]);
      bg.add(menuObjectColors[i]);
    }

    menuObjectDelete=new JMenuItem("Delete Object", KeyEvent.VK_D);
    menuObjectDelete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
    menuObjectDelete.addActionListener(this);
    menu.add(menuObjectDelete);

    menuObjectRename=new JMenuItem("Rename Object", KeyEvent.VK_R);
    menuObjectRename.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, 0));
    menuObjectRename.addActionListener(this);
    menu.add(menuObjectRename);

    menu=new JMenu("Line");
    menu.setMnemonic(KeyEvent.VK_L);
    menubar.add(menu);

    menuLineNew=new JMenuItem("New Line", KeyEvent.VK_N);
    menuLineNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
    menuLineNew.addActionListener(this);
    menu.add(menuLineNew);

    menuLinePoint=new JMenuItem("Node at playerpos", KeyEvent.VK_N);
    menuLinePoint.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, 0));
    menuLinePoint.addActionListener(this);
    menu.add(menuLinePoint);

    menuLineDelete=new JMenuItem("Delete Node", KeyEvent.VK_D);
    menuLineDelete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, 0));
    menuLineDelete.addActionListener(this);
    menu.add(menuLineDelete);

    menuLineFlatten=new JMenuItem("Flatten to Player Z", KeyEvent.VK_Z);
    menuLineFlatten.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, 0));
    menuLineFlatten.addActionListener(this);
    menu.add(menuLineFlatten);
    
    menuLineFirst=new JMenuItem("Goto First Node", KeyEvent.VK_F);
    menuLineFirst.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, 0));
    menuLineFirst.addActionListener(this);
    menu.add(menuLineFirst);

    menuLineLast=new JMenuItem("Goto Last Node", KeyEvent.VK_L);
    menuLineLast.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, 0));
    menuLineLast.addActionListener(this);
    menu.add(menuLineLast);

    menuLineFilled=new JCheckBoxMenuItem("Filled");
    menuLineFilled.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, 0));
    menuLineFilled.addItemListener(this);
    menu.add(menuLineFilled);

    menu=new JMenu("Point");
    menu.setMnemonic(KeyEvent.VK_P);
    menubar.add(menu);

    menuPointNew=new JMenuItem("New Point at playerpos", KeyEvent.VK_N);
    menuPointNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, 0));
    menuPointNew.addActionListener(this);
    menu.add(menuPointNew);

    GridBagLayout gb=new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();

    getContentPane().setLayout(gb);
    c.fill = GridBagConstraints.BOTH;
    
    c.gridx = 0;
    c.gridy = 0;
    c.weightx = 1.0;
    c.weighty = 1.0;
    c.gridwidth = 1;
    c.gridheight = 1;

    con = new Connection(this);
    gb.setConstraints(con, c);
    getContentPane().add(con);

    c.gridy = 1;
    c.weighty = 8.0;
    map = new Map(this);
    gb.setConstraints(map, c);
    getContentPane().add(map);

    addKeyListener(this);

    lastfile = null;
  }

  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == menuFileNew) {
      map.wipe();
      lastfile = null;
    } else if (e.getSource() == menuFileOpen) {
      JFileChooser chooser=new JFileChooser();
      chooser.setSelectedFile(lastfile);
      int retVal = chooser.showOpenDialog(this);
      if (retVal == JFileChooser.APPROVE_OPTION) {
         lastfile=chooser.getSelectedFile();
         map.read(lastfile);
         map.clean();
      }
    } else if (e.getSource() == menuFileSave) {
      JFileChooser chooser=new JFileChooser();
      chooser.setSelectedFile(lastfile);
      int retVal = chooser.showSaveDialog(this);
      if (retVal == JFileChooser.APPROVE_OPTION) {
         lastfile=chooser.getSelectedFile();
         map.write(lastfile);
      }
    } else if (e.getSource() == menuFileExit) {
      con.disconnect();
      System.exit(0);
    } else if (e.getSource() == menuLinkConnect) {
      if (con.connect()) {
        menuLinkDisconnect.setEnabled(true);
      }
    } else if (e.getSource() == menuLinkDisconnect) {
      con.disconnect();
      menuLinkDisconnect.setEnabled(false);
    } else if (e.getSource() == menuMapClean) {
      map.clean();
    } else if (e.getSource() == menuMapSimple) {
      map.simplify();
    } else if (e.getSource() == menuObjectRename) {
      map.renameObject();
    } else if (e.getSource() == menuObjectDelete) {
      map.deleteObject();
    } else if (e.getSource() == menuLineNew) {
      map.lineNew();
    } else if (e.getSource() == menuLinePoint) {
      map.linePoint();
    } else if (e.getSource() == menuLineDelete) {
      map.lineDelete();
    } else if (e.getSource() == menuLineFlatten) {
      map.lineFlatten();
    } else if (e.getSource() == menuLineFirst) {
      map.lineFirst();
    } else if (e.getSource() == menuLineLast) {
      map.lineLast();
    } else if (e.getSource() == menuPointNew) {
      map.pointNew();
    }
    repaint();
  }

  public void itemStateChanged(ItemEvent e) {
    for(int i=0;i<colors.length;i++) {
      if ((e.getSource() == menuObjectColors[i]) && menuObjectColors[i].isSelected()) {
         color=colors[i];
         map.setColor(color);
      }
    }
    if (e.getSource() == menuLineFilled) {
      map.setFilled(menuLineFilled.isSelected());
    }
    repaint();
  }

  public void keyPressed(KeyEvent e) {
  }

  public void keyReleased(KeyEvent e) {
    int keyCode=e.getKeyCode();
    switch (keyCode) {
      case KeyEvent.VK_RIGHT:
        map.move(0);
        break;
      case KeyEvent.VK_UP:
        map.move(1);
        break;
      case KeyEvent.VK_LEFT:
        map.move(2);
        break;
      case KeyEvent.VK_DOWN:
        map.move(3);
        break;
      case KeyEvent.VK_HOME:
        map.home();
        break;
      case KeyEvent.VK_PAGE_UP:
        map.zoom(-1);
        break;
      case KeyEvent.VK_PAGE_DOWN:
        map.zoom(1);
        break;
      default:
        return;
    }
    repaint();
  }

  public void keyTyped(KeyEvent e) {
  }

  public void updateSelected() {
    MapElement e=map.getSelected();
    if (e == null) 
      return;

    if (e instanceof MapElementLine) {
       MapElementLine l=(MapElementLine) e;
       menuLineFilled.setSelected(l.getFilled());
    }

    for(int i=0;i<colors.length;i++) {
       if (colors[i].equals(e.getColor())) {
         menuObjectColors[i].setSelected(true);
       }
     }
  }

  public void updateConnection() {
    map.repaint();
  }
 
  public static String colorToString(Color c) {
    for(int i=0;i<colors.length;i++) {
      if (c.equals(colors[i]))
        return colornames[i].toLowerCase();
    }
    return "white";
  }

  public static Color stringToColor(String s) {
    s=s.toLowerCase();
    for(int i=0;i<colors.length;i++) {
      if (s.equals(colornames[i].toLowerCase())) 
        return colors[i];
    }
    return Color.white;
  }

  public void tryConnect(String v) {
    con.connect(v);
  }

  public void tryLoad(String v) {
     lastfile = new File(v);
     if (! lastfile.exists()) {
       lastfile=null;
       return;
     }
     map.read(lastfile);
     map.clean();
  }

  public void setSave(String v) {
     lastfile = new File(v);
  }

  public static void main(String args[]) {
    MapEditor frm=new MapEditor();
    frm.addWindowListener(new WindowAdapter() {
                                public void windowClosing(WindowEvent e) {
                                System.exit(0);
                                }
                        });
    frm.show();
    if (args.length == 3) {
      frm.tryConnect(args[0]);
      frm.tryLoad(args[1]);
      frm.setSave(args[2]);
    }
  }
}
