/*      	 
 * Copyright 2002 Slicer/HackersQuest (slicer@hackersquest.org)
 *
 * This file is part of Odin's Eye.
 *
 * Odin's Eye is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Odin's Eye is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public abstract class MapElement implements Comparable {
  Color col;
  String title;
  
  public MapElement() {
    col = Color.white;
    title = "Element";
  }

  public void setColor(Color c) {
    col=c;
  }

  public Color getColor() {
    return col;
  }

  public void setTitle(String t) {
    title=t;
  }

  public String getTitle() {
    return title;
  }

  public static String join(java.util.List lst) {
    String n=new String();
    Object[] l=lst.toArray();
    if (l.length == 0)
      return n;
    n+=l[0].toString();
    for(int i=1;i<l.length;i++) {
      n+=","+l[i].toString();
    }
    return n;
  }

  public abstract void paint(Graphics2D g, Map m);
  public abstract boolean visible(Rectangle Bounds);
  public abstract double clickAt(Point p);
  public abstract int getZ();

  public String toString() {
    ArrayList al=new ArrayList();
    al.add(new String("N"));
    al.add(MapEditor.colorToString(col));
    return join(al);
  }

  public boolean fromString(String s[]) {
    if (s.length < 3)
      return false;
    title=s[1];
    col=MapEditor.stringToColor(s[2]);    
    return true;
  }

  public static MapElement fromString(String s) {
    String v[]=s.split(",");
    MapElementPoint p;
    MapElementLine l;

    if (v[0].equals("P")) {
      p=new MapElementPoint();
      if (p.fromString(v))
        return p;
    } else if (v[0].equals("M") || v[0].equals("F")) {
      l=new MapElementLine();
      if (! l.fromString(v))
        return null;
      if (v[0].equals("F")) 
        l.setFilled(true);
      return l;
    }
    return null;
  }

  public int compareTo(Object o) {
    MapElement e=(MapElement) o;
    int myz=getZ();
    int otherz=e.getZ();
    if (myz <= otherz)
      return -1;
    return 1;
  }

  public boolean cleanAndValidate() {
    if ((col == null) || (title == null) || (title.length() < 1)) 
      return false;
    return true;
  }
}
