/*      	 
 * Copyright 2002 Slicer/HackersQuest (slicer@hackersquest.org)
 *
 * This file is part of Odin's Eye.
 *
 * Odin's Eye is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Odin's Eye is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class MapElementPoint extends MapElement {
  int x, y, z;
  int pos[];

  public MapElementPoint() {
    super();
    x=y=z=0;
    title = "Point";
    pos = new int[2];
  }

  public MapElementPoint(int xv, int yv, int zv) {
    this();
    x=xv;
    y=yv;
    z=zv;
  }

  public void paint(Graphics2D g, Map m) {
    int xv;
    int yv;
    int zv;
    int w;

    xv=m.transx(x, y);
    yv=m.transy(x, y);

    g.setColor(col);    
    
    w = 3;

    if (m.getSelected() == this) {
      w = 5;
    }

    g.fillOval(xv-w,yv-w,w*2+1,w*2+1);
    g.drawString(title, xv+4,yv);
  }

  public boolean visible(Rectangle bounds) {
    return bounds.contains(x, y);
  }

  public double clickAt(Point p) {
    return p.distance(x,y);
  }

  public String toString() {
    ArrayList al=new ArrayList();
    al.add(new String("P"));
    al.add(title);
    al.add(MapEditor.colorToString(col));
    al.add(String.valueOf(x));
    al.add(String.valueOf(y));
    al.add(String.valueOf(z));
    return join(al);
  }

  public boolean fromString(String s[]) {
    if (s.length != 6)
      return false;

    if (! super.fromString(s))
      return false;
    try {
      x=Math.abs(Integer.valueOf(s[3]).intValue());
      y=Math.abs(Integer.valueOf(s[4]).intValue());
      z=Math.abs(Integer.valueOf(s[5]).intValue()); 
    } catch (NumberFormatException e) {
      return false;
    }
    return true;
  }

  public int getZ() {
    return z;
  }
}
